import { Signal } from "@hmans/signal";
import { RegisteredEntity } from ".";
import { ComponentName, EntityWith, IEntity } from "./World";
/**
 * A query is an array of component names.
 */
export declare type Query<T extends IEntity> = ComponentName<T>[];
export declare type ArchetypeEntity<TEntity extends IEntity, TQuery extends Query<TEntity> = Query<TEntity>> = EntityWith<TEntity, TQuery[number]>;
export declare class Archetype<TEntity extends IEntity, TQuery extends Query<TEntity> = Query<TEntity>> {
    query: TQuery;
    /** A list of entities belonging to this archetype. */
    entities: ArchetypeEntity<RegisteredEntity<TEntity>, TQuery>[];
    get [Symbol.iterator](): () => IterableIterator<ArchetypeEntity<RegisteredEntity<TEntity>, TQuery>>;
    /** Returns the first entity within this archetype. */
    get first(): ArchetypeEntity<RegisteredEntity<TEntity>, TQuery> | null;
    /** Listeners on this event are invoked when an entity is added to this archetype's index. */
    onEntityAdded: Signal<ArchetypeEntity<RegisteredEntity<TEntity>, TQuery>, (payload: ArchetypeEntity<RegisteredEntity<TEntity>, TQuery>) => void>;
    /** Listeners on this event are invoked when an entity is removed from this archetype's index. */
    onEntityRemoved: Signal<RegisteredEntity<TEntity>, (payload: RegisteredEntity<TEntity>) => void>;
    constructor(query: TQuery);
    indexEntity(entity: RegisteredEntity<TEntity>): void;
    removeEntity(entity: RegisteredEntity<TEntity>): void;
}
